class PostcoderEmail{
  
  constructor(config) {
    this.config = config
    this.init()
  }

  init() {

    this.endpoint = 'https://ws.postcoder.com/pcw/' + this.config.apikey + '/emailaddress/'
    
    this.input = document.querySelector(this.config.searchterm)

    // add blur event listener to the email field, to call the search function
    this.input.addEventListener('blur', this.search)

    // create an empty div for the tick/cross and place it in the page
    this.successindicator = document.querySelector(this.config.successindicator)

    // create an empty div for the alternative email messasge and place it hidden in the page
    this.alternative = document.createElement('div')
    this.alternative.style.display = 'none'
    this.input.parentNode.insertBefore(this.alternative, this.input.nextSibling)

    // add a result container to the page, delete when not required.
    this.resultcontainer = document.createElement('pre')
    this.input.closest('form').appendChild(this.resultcontainer)

  }

  search = event => {

    //remove previous email validity indicator
    this.successindicator.classList.remove('isvalid','notvalid')

    //hide the alternaive email message
    this.alternative.style.display = 'none'

    if(this.input.value){

      let self = this
      let searchvalue = encodeURIComponent(this.input.value)

      // fetch the json formatted result from Postcoder and pass it to processResult
      fetch(this.endpoint + searchvalue)
        .then( response => {
          if (!response.ok) { 
            throw response 
          }
          return response.json()
        })
        .then( json => {
          self.processResult(json)
        })
        .catch( err => {

          if(typeof err.text === 'function'){
            err.text().then( errorMessage => {
              console.log('Postcoder request error ' + err.status + ' : ' + errorMessage)
            })
          }else{
            console.log(err)
          }
      })
    }
  }

  processResult = result =>{

    // display the email validity indicator
    if(result.valid === true){
      this.successindicator.classList.add('isvalid')
    }else{
      this.successindicator.classList.add('notvalid')
    }

    // display any alternative email suggestion
    if(result.alternative){
      this.alternative.innerHTML = 'Did you mean ' + result.alternative + '?'
      this.alternative.style.display = 'block'
    }

    // show result on page, delete when not required.
    this.resultcontainer.innerHTML = "Result: \n\n" + JSON.stringify( result, null, 4 )

  }

}